#pragma once
#include "tinlib.h"

namespace tingui {
	using namespace System;
	using namespace System::ComponentModel;
	using namespace System::Collections;
	using namespace System::Windows::Forms;
	using namespace System::Data;
	using namespace System::Drawing;
	using namespace tinlib;

	/// <summary>
	/// Summary for tin_form
	///
	/// WARNING: If you change the name of this class, you will need to change the
	///          'Resource File Name' property for the managed resource compiler tool
	///          associated with all .resx files this class depends on.  Otherwise,
	///          the designers will not be able to interact properly with localized
	///          resources associated with this form.
	/// </summary>
	public ref class tin_form : public System::Windows::Forms::Form
	{
	public:
		tin_form(void)
		{
			InitializeComponent();
			//
			//TODO: Add the constructor code here
			//
		}

		void convert_string(String^ managed_string,char *unmanaged_string){
			// this method from http://www.winterdom.com/mcppfaq/archives/000111.html
			using namespace System::Runtime::InteropServices;
			const char* str = 
				(const char*)(Marshal::StringToHGlobalAnsi(managed_string)).ToPointer();
			strcpy(unmanaged_string,str);
			// free string
			Marshal::FreeHGlobal(IntPtr((void*)str));
		}
		void print_triangles(tinner_class ^tinner)
		/* *******************************************************************
		*  Print out all the triangles created by tinning.
		* ******************************************************************* */
		{
			ulong vdex0,vdex1,vdex2;// indexes of the points of the triangle corners
			double x0,y0,z0, x1,y1,z1, x2,y2,z2;

			System::String^ outstr = System::String::Empty;/* initialize the output string */
			
			System::Console::WriteLine("tinner->num_triangles:{0}.",tinner->num_triangles);

			/* tcnt is the triangle index */
			for (ulong tdex=0;tdex<tinner->num_triangles;tdex++){
				/* v0,v1, and v2 return the indexes of each triangle corner, as found in the point array */
				if(tinner->get_triangle_indexes(tdex, vdex0, vdex1, vdex2)){
					/* so now that we have the vertex indexes, look up the actual coordinates */
					tinner->get_point(vdex0, x0,y0,z0);
					tinner->get_point(vdex1, x1,y1,z1);
					tinner->get_point(vdex2, x2,y2,z2);

					/* now just put the results into the output string */
					outstr += System::String::Format("triangle {0}: ",tdex);
					outstr += System::String::Format("(x:{0} y:{1} z:{2}) ",x0,y0,z0);
					outstr += System::String::Format("(x:{0} y:{1} z:{2}) ",x1,y1,z1);
					outstr += System::String::Format("(x:{0} y:{1} z:{2}) ",x2,y2,z2);
					outstr += Environment::NewLine;
				}else{/* if it returned false, then we went out of range of the list of triangles. */
					System::Console::WriteLine("Error, no triangle at index:%{0}.",tdex);
				}
			}/* end for loop. */
			this->output_textbox->Text = outstr;
		}/* print_triangles */
		
		void tin_a_file(char *fname)
		/* *******************************************************************
		*  This shows the essentials of using the TIN library.
		* ******************************************************************* */
		{
			tinner_class ^tinner = gcnew tinner_class();

			/* very big search radius, to produce a fully convex outline of the data. */
			double max_triangle_radius = 100000000000000000.0;

			tin_bool success;
			if (true){
				/* read in all the survey xyz points. */
				success = tinner->tin_read_xyzs(fname);
			}else{
				/* alternately, here's a demo of how to load your own points without a file: */
				generate_random_points(tinner,1000);
				success = true;
			}
			if (success){
				System::Console::WriteLine("%{0} survey xyz points.",tinner->points->NumPoints);

				/* Create the TIN. */
				System::Console::WriteLine("About to tin_create.");

				/* Generate the TIN mesh from the data. */
				tinner->generate_tin(max_triangle_radius);

				MessageBox::Show("TIN generated.  Click to save the output.");

				/* show off all the triangles created. */
				print_triangles(tinner);

				/* save the triangles and outline to text files. */
				System::Console::WriteLine("Saving triangles to triangles.txt...");
				tinner->save_triangles("triangles.txt");

				System::Console::WriteLine("Saving outline to outline.txt...");
				tinner->save_outline("outline.txt");
			
				MessageBox::Show("TIN created and stored in triangles.txt.  The outline is in outline.txt.");
			}

			/* dispose of all of the tinner's internal structures. */
			System::Console::WriteLine("About to delete the tinner.");
			delete tinner;

			System::Console::WriteLine("Done."); /* Bye! */
		}/* tin_a_file */

		void generate_random_points(tinner_class ^tinner,int numpoints)
		/* *******************************************************************
		*  Create a field of random points for testing.
		* ******************************************************************* */
		{
			double xval,yval,zval;
			time_t randseed=time(0); srand(randseed);// randomize by time

			/* important: first allocate the memory space that will hold the points */
			tinner->allocate_points(numpoints);

			for(int index=0;index<numpoints;index++){
				xval = 100.0*((double)(rand()%12345))/12345.6;
				yval = 100.0*((double)(rand()%12345))/12345.6;
				zval = ((double)(rand()%12345))/12345.6;

				tinner->set_point(index,xval,yval,zval);// this line is like loading an array: index, then xyz values.
			}
		}/* generate_random_points */

	protected:
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		~tin_form()
		{
			if (components)
			{
				delete components;
			}
		}
	private: System::Windows::Forms::Button^  launch_button;
	private: System::Windows::Forms::TextBox^  output_textbox;
	private: System::Windows::Forms::OpenFileDialog^  openfile_dialog;

	protected: 

	private:
		/// <summary>
		/// Required designer variable.
		/// </summary>
		System::ComponentModel::Container ^components;

#pragma region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		void InitializeComponent(void)
		{
			this->launch_button = (gcnew System::Windows::Forms::Button());
			this->output_textbox = (gcnew System::Windows::Forms::TextBox());
			this->openfile_dialog = (gcnew System::Windows::Forms::OpenFileDialog());
			this->SuspendLayout();
			// 
			// launch_button
			// 
			this->launch_button->Dock = System::Windows::Forms::DockStyle::Top;
			this->launch_button->Font = (gcnew System::Drawing::Font(L"Microsoft Sans Serif", 14, System::Drawing::FontStyle::Regular, System::Drawing::GraphicsUnit::Point, 
				static_cast<System::Byte>(0)));
			this->launch_button->Location = System::Drawing::Point(0, 0);
			this->launch_button->Name = L"launch_button";
			this->launch_button->Size = System::Drawing::Size(491, 62);
			this->launch_button->TabIndex = 0;
			this->launch_button->Text = L"Click here to load your xyz.txt file.";
			this->launch_button->UseVisualStyleBackColor = true;
			this->launch_button->Click += gcnew System::EventHandler(this, &tin_form::launch_button_Click);
			// 
			// output_textbox
			// 
			this->output_textbox->Dock = System::Windows::Forms::DockStyle::Fill;
			this->output_textbox->Location = System::Drawing::Point(0, 62);
			this->output_textbox->Multiline = true;
			this->output_textbox->Name = L"output_textbox";
			this->output_textbox->Size = System::Drawing::Size(491, 447);
			this->output_textbox->TabIndex = 1;
			// 
			// openfile_dialog
			// 
			this->openfile_dialog->FileName = L"XYZ.txt";
			this->openfile_dialog->FileOk += gcnew System::ComponentModel::CancelEventHandler(this, &tin_form::openfile_dialog_FileOk);
			// 
			// tin_form
			// 
			this->AutoScaleDimensions = System::Drawing::SizeF(6, 13);
			this->AutoScaleMode = System::Windows::Forms::AutoScaleMode::Font;
			this->ClientSize = System::Drawing::Size(491, 509);
			this->Controls->Add(this->output_textbox);
			this->Controls->Add(this->launch_button);
			this->Name = L"tin_form";
			this->Text = L"TIN form";
			this->ResumeLayout(false);
			this->PerformLayout();

		}
#pragma endregion
	private: System::Void launch_button_Click(System::Object^  sender, System::EventArgs^  e) {
				 openfile_dialog->ShowDialog(this);
			 }

	private: System::Void openfile_dialog_FileOk(System::Object^  sender, System::ComponentModel::CancelEventArgs^  e) {
				 char fname[600];
				 convert_string(openfile_dialog->FileName,fname);
				 tin_a_file(fname);
			 }
	};
}

