
// tinlib.h
// tinlib.h : main header file for the tinlib DLL
/* **************************************************************************

  TIN.H, a library to create Triangulated Irregular Networks
from xyz point data.

Copyright (C) 2000 John Conner Atkeson III

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

For further information, contact:
John Conner Atkeson III
jcatkeson at gmail dot com
http://jca3.freeshell.org
3959 Persimmon Drive #103 Fairfax Va. 22031

************************************************************************** */

#ifndef tinlib_h
#define tinlib_h 1

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
// #include <cmath> // for trigonometry functions
#include <string.h>
#include <time.h>
#include "rosetta.h"
#include "flexray.h"
#include "space.h"

// using System::Runtime::InteropServices;

#pragma once

#define byradius 1

// static double tin_radius=1.0/fudge;

/* possible states a line segment can have while we create a TIN. */
typedef enum{alive,dead,outer} lstateT;


#include "resource.h" // main symbols
using namespace System;

// CtinlibApp
// See tinlib.cpp for the implementation of this class
//
typedef struct lineT *lineptrT;
typedef struct lineT{
  intdex pdex2,pdex1;/* indexes to endpoints */
  lstateT state;
}lineT;
/* an array of pointers to lines, nothing more. */
typedef struct linelistT{
  int nlines;
  lineT **L;
}linelistT;

/* triangle type. */
typedef struct triT{
  intdex vtex[3];/* indexes of 3 vertex points, in points array. */
}triT;

/* complete triangle type, no dereferencing required. */
typedef struct full_triT{
  double vtex[3][3];/* 3 xyz points */
  double slope[2];/* x and y slopes */
}full_triT;

/* Bricks in the loop wall. */
typedef struct brickT *brickptrT;
typedef struct brickT{
  brickptrT prev,next;
  lineT *linptr;
}brickT;

namespace tinlib {

	public ref class tinner_class
    /* *******************************************************************
    *  Serves the purpose of global variables, but neater.
    * ******************************************************************* */
	{
    public:
      plist_class *points;
      VectT *sort;
      VectT *paramv;

      /* top of flexray tree of lines in loopwall. */
      flexelT* line_tree;
      /* running list of lines in boundary of growing lattice */
      brickptrT loop_wall;

      /* tree of triangles we create. */
      uint32 num_triangles;
      flexelT* tri_tree;
  
      /* _roughly_ how long a triangle`s legs are allowed to be. */
      double border_segment_max;

      /* methods */
      tin_bool tinner_class::tin_read_xyzs(char *filename);
      void tinner_class::generate_tin(double new_border_segment_max);
      uint32 tinner_class::create_random(int numpoints);/* Generate random points for testing. */

	  tin_bool tinner_class::get_triangle_full(full_triT *tri,uint32 tri_num);
	  tin_bool tinner_class::get_triangle_indexes(ulong tdex, ulong% v0, ulong% v1, ulong% v2);
	  tin_bool tinner_class::get_triangle_verts(int tdex, 
		  double *x0,double *y0,double *z0, 
		  double *x1,double *y1,double *z1, 							   
		  double *x2,double *y2,double *z2);

	  void tinner_class::allocate_points(int new_numpoints);
	  void tinner_class::set_point(ulong pdex,double xval,double yval,double zval);
	  void tinner_class::get_point(ulong pdex, double% xval, double% yval, double% zval);
	  uint32 tinner_class::get_num_points();
#if 0	   
void tinner_class::get_point2(ulong pdex,[Paramflag::Out] double xval, double% yval, double% zval)
/* *******************************************************************
* ******************************************************************* */
{
	
	XYZType *xyz = &(points->P[pdex]);
	xval = xyz->Loc[0]; yval = xyz->Loc[1]; zval = xyz->Loc[2];
}/* get_point */
#endif
      void tinner_class::save_outline(char *filename);
      void tinner_class::draw_triangles();
      void tinner_class::save_triangles(char *filename);
      void tinner_class::save_tinfile(char *filename);

	  void tinner_class::clear();

	  /* constructor and destructor */
	  tinner_class::tinner_class();
      tinner_class::~tinner_class();

    private:
      void tinner_class::store_triangle(intdex pdex0,intdex pdex1,intdex pdex2);
      tin_bool tinner_class::MakeFirst();
      tin_bool tinner_class::FindClosest(intdex CenterDex,intdex *NbrDex,LogicType *Filter);
      tin_bool tinner_class::FindThird(intdex pt0,intdex pt1,intdex *MinSubjIndex,LogicType *Filter);
      void tinner_class::PushLine(lineT *newline);
	  void tinner_class::PopLine();
	  // void tinner_class::tin_init();
	};//tinner_class
	
    void draw_triangle(plist_class *points,intdex pdex1,intdex pdex2,intdex pdex3,int color);
    void DrawLoop(brickptrT loop_wall,plist_class *points,int color);
}//namespace tinlib 

#endif // ifndef tinlib_h

